@@ -27,7 +27,6 @@ import time
 
 import lz4.block
 import numpy
-import six
 
 from gnocchi import calendar
 
@@ -78,10 +77,10 @@ def make_timeseries(timestamps, values):
     This array specifies correctly the data types, which is important for
     Numpy to operate fastly.
     """
-    l = len(timestamps)
-    if l != len(values):
+    length = len(timestamps)
+    if length != len(values):
         raise ValueError("Timestamps and values must have the same length")
-    arr = numpy.zeros(l, dtype=TIMESERIES_ARRAY_DTYPE)
+    arr = numpy.zeros(length, dtype=TIMESERIES_ARRAY_DTYPE)
     arr['timestamps'] = timestamps
     arr['values'] = values
     return arr
@@ -129,15 +128,19 @@ class GroupedTimeSeries(object):
         self.tstamps, self.counts = numpy.unique(self.indexes,
                                                  return_counts=True)
 
+    @staticmethod
+    def _sum(values, counts):
+        return numpy.bincount(
+            numpy.repeat(numpy.arange(counts.size), counts), weights=values)
+
     def mean(self):
-        series = self.sum()
-        series['values'] /= self.counts
-        return series
+        return make_timeseries(
+            self.tstamps,
+            self._sum(self._ts['values'], self.counts) / self.counts)
 
     def sum(self):
-        return make_timeseries(self.tstamps, numpy.bincount(
-            numpy.repeat(numpy.arange(self.counts.size), self.counts),
-            weights=self._ts['values']))
+        return make_timeseries(self.tstamps,
+                               self._sum(self._ts['values'], self.counts))
 
     def min(self):
         ordered = self._ts['values'].argsort()
@@ -165,9 +168,9 @@ class GroupedTimeSeries(object):
                            self._ts['values'][ordered][mid_ceil]) / 2.0)
 
     def std(self):
-        mean_ts = self.mean()
+        values = self._sum(self._ts['values'], self.counts) / self.counts
         diff_sq = numpy.square(self._ts['values'] -
-                               numpy.repeat(mean_ts['values'], self.counts))
+                               numpy.repeat(values, self.counts))
         bin_sum = numpy.bincount(
             numpy.repeat(numpy.arange(self.counts.size), self.counts),
             weights=diff_sq)
@@ -192,8 +195,8 @@ class GroupedTimeSeries(object):
         ordered = numpy.lexsort((self._ts['values'], self.indexes))
         min_pos = numpy.cumsum(self.counts) - self.counts
         real_pos = min_pos + (self.counts - 1) * (q / 100)
-        floor_pos = numpy.floor(real_pos).astype(numpy.int, copy=False)
-        ceil_pos = numpy.ceil(real_pos).astype(numpy.int, copy=False)
+        floor_pos = numpy.floor(real_pos).astype(numpy.integer, copy=False)
+        ceil_pos = numpy.ceil(real_pos).astype(numpy.integer, copy=False)
         values = (
             self._ts['values'][ordered][floor_pos] * (ceil_pos - real_pos) +
             self._ts['values'][ordered][ceil_pos] * (real_pos - floor_pos))
@@ -228,7 +231,7 @@ class TimeSerie(object):
         self.ts = ts
 
     def __iter__(self):
-        return six.moves.zip(self.ts['timestamps'], self.ts['values'])
+        return zip(self.ts['timestamps'], self.ts['values'])
 
     @classmethod
     def from_data(cls, timestamps=None, values=None):
@@ -236,7 +239,7 @@ class TimeSerie(object):
 
     def __eq__(self, other):
         return (isinstance(other, TimeSerie) and
-                numpy.array_equal(self.ts,  other.ts))
+                numpy.array_equal(self.ts, other.ts))
 
     def __getitem__(self, key):
         if isinstance(key, numpy.datetime64):
@@ -307,9 +310,8 @@ class TimeSerie(object):
 
     @staticmethod
     def _compress(payload):
-        # FIXME(jd) lz4 > 0.9.2 returns bytearray instead of bytes. But Cradox
-        # does not accept bytearray but only bytes, so make sure that we have a
-        # byte type returned.
+        # FIXME(jd) lz4 > 0.9.2 returns bytearray instead of bytes but we need
+        # a byte type returned.
         return memoryview(lz4.block.compress(payload)).tobytes()
 
 
@@ -408,34 +410,34 @@ class BoundTimeSerie(TimeSerie):
         now = numpy.datetime64("2015-04-03 23:11")
         timestamps = numpy.sort(numpy.array(
             [now + numpy.timedelta64(random.randint(1000000, 10000000), 'us')
-             for i in six.moves.range(points)]))
+             for i in range(points)]))
 
         print(cls.__name__)
         print("=" * len(cls.__name__))
 
         for title, values in [
-                ("Simple continuous range", six.moves.range(points)),
+                ("Simple continuous range", range(points)),
                 ("All 0", [float(0)] * points),
                 ("All 1", [float(1)] * points),
                 ("0 and 1", [0, 1] * (points // 2)),
                 ("1 and 0 random",
                  [random.randint(0, 1)
-                  for x in six.moves.range(points)]),
+                  for x in range(points)]),
                 ("Small number random pos/neg",
                  [random.randint(-100000, 10000)
-                  for x in six.moves.range(points)]),
+                  for x in range(points)]),
                 ("Small number random pos",
-                 [random.randint(0, 20000) for x in six.moves.range(points)]),
+                 [random.randint(0, 20000) for x in range(points)]),
                 ("Small number random neg",
-                 [random.randint(-20000, 0) for x in six.moves.range(points)]),
-                ("Sin(x)", list(map(math.sin, six.moves.range(points)))),
+                 [random.randint(-20000, 0) for x in range(points)]),
+                ("Sin(x)", list(map(math.sin, range(points)))),
                 ("random ", [random.random()
-                             for x in six.moves.range(points)]),
+                             for x in range(points)]),
         ]:
             print(title)
             ts = cls.from_data(timestamps, values)
             t0 = time.time()
-            for i in six.moves.range(serialize_times):
+            for i in range(serialize_times):
                 s = ts.serialize()
             t1 = time.time()
             print("  Serialization speed: %.2f MB/s"
@@ -444,7 +446,7 @@ class BoundTimeSerie(TimeSerie):
             print("   Bytes per point: %.2f" % (len(s) / float(points)))
 
             t0 = time.time()
-            for i in six.moves.range(serialize_times):
+            for i in range(serialize_times):
                 cls.unserialize(s, ONE_SECOND, 1)
             t1 = time.time()
             print("  Unserialization speed: %.2f MB/s"
@@ -545,7 +547,10 @@ class SplitKey(object):
         return str(float(self))
 
     def __float__(self):
-        return datetime64_to_epoch(self.key)
+        f = datetime64_to_epoch(self.key)
+        if isinstance(f, numpy.float64):
+            return f.item()
+        return f
 
     def __repr__(self):
         return "<%s: %s / %s>" % (self.__class__.__name__,
@@ -634,7 +639,7 @@ class AggregatedTimeSerie(TimeSerie):
             round_timestamp(self.timestamps, freq),
             return_counts=True)
         start = 0
-        for key, count in six.moves.zip(keys, counts):
+        for key, count in zip(keys, counts):
             end = start + count
             yield (SplitKey(key, self.aggregation.granularity),
                    AggregatedTimeSerie(self.aggregation, self[start:end]))
@@ -677,7 +682,7 @@ class AggregatedTimeSerie(TimeSerie):
     @staticmethod
     def is_compressed(serialized_data):
         """Check whatever the data was serialized with compression."""
-        return six.indexbytes(serialized_data, 0) == ord("c")
+        return serialized_data[0] == ord("c")
 
     @classmethod
     def unserialize(cls, data, key, aggregation):
@@ -701,7 +706,7 @@ class AggregatedTimeSerie(TimeSerie):
                                          count=nb_points)
                     x = numpy.frombuffer(
                         uncompressed, dtype='<d',
-                        offset=nb_points*cls.COMPRESSED_TIMESPAMP_LEN)
+                        offset=nb_points * cls.COMPRESSED_TIMESPAMP_LEN)
                 except ValueError:
                     raise InvalidData()
                 y = numpy.cumsum(y * key.sampling) + key.key
@@ -769,7 +774,7 @@ class AggregatedTimeSerie(TimeSerie):
         first = self.first  # NOTE(jd) needed because faster
         e_offset = int((self.last - first) / offset_div) + 1
 
-        locs = numpy.zeros(self.timestamps.size, dtype=numpy.int)
+        locs = numpy.zeros(self.timestamps.size, dtype=numpy.integer)
         locs[1:] = numpy.cumsum(numpy.diff(self.timestamps)) / offset_div
 
         # Fill everything with zero and set
@@ -810,29 +815,29 @@ class AggregatedTimeSerie(TimeSerie):
         now = numpy.datetime64("2015-04-03 23:11")
         timestamps = numpy.sort(numpy.array(
             [now + i * sampling
-             for i in six.moves.range(points)]))
+             for i in range(points)]))
 
         print(cls.__name__)
         print("=" * len(cls.__name__))
 
         for title, values in [
-                ("Simple continuous range", six.moves.range(points)),
+                ("Simple continuous range", range(points)),
                 ("All 0", [float(0)] * points),
                 ("All 1", [float(1)] * points),
                 ("0 and 1", [0, 1] * (points // 2)),
                 ("1 and 0 random",
                  [random.randint(0, 1)
-                  for x in six.moves.range(points)]),
+                  for x in range(points)]),
                 ("Small number random pos/neg",
                  [random.randint(-100000, 10000)
-                  for x in six.moves.range(points)]),
+                  for x in range(points)]),
                 ("Small number random pos",
-                 [random.randint(0, 20000) for x in six.moves.range(points)]),
+                 [random.randint(0, 20000) for x in range(points)]),
                 ("Small number random neg",
-                 [random.randint(-20000, 0) for x in six.moves.range(points)]),
-                ("Sin(x)", list(map(math.sin, six.moves.range(points)))),
+                 [random.randint(-20000, 0) for x in range(points)]),
+                ("Sin(x)", list(map(math.sin, range(points)))),
                 ("random ", [random.random()
-                             for x in six.moves.range(points)]),
+                             for x in range(points)]),
         ]:
             print(title)
             serialize_times = 50
@@ -840,7 +845,7 @@ class AggregatedTimeSerie(TimeSerie):
             ts = cls.from_data(aggregation, timestamps, values)
             t0 = time.time()
             key = ts.get_split_key()
-            for i in six.moves.range(serialize_times):
+            for i in range(serialize_times):
                 e, s = ts.serialize(key, compressed=False)
             t1 = time.time()
             print("  Uncompressed serialization speed: %.2f MB/s"
@@ -849,7 +854,7 @@ class AggregatedTimeSerie(TimeSerie):
             print("   Bytes per point: %.2f" % (len(s) / float(points)))
 
             t0 = time.time()
-            for i in six.moves.range(serialize_times):
+            for i in range(serialize_times):
                 cls.unserialize(s, key, 'mean')
             t1 = time.time()
             print("  Unserialization speed: %.2f MB/s"
@@ -857,7 +862,7 @@ class AggregatedTimeSerie(TimeSerie):
                       / ((t1 - t0) / serialize_times)) / (1024.0 * 1024.0)))
 
             t0 = time.time()
-            for i in six.moves.range(serialize_times):
+            for i in range(serialize_times):
                 o, s = ts.serialize(key, compressed=True)
             t1 = time.time()
             print("  Compressed serialization speed: %.2f MB/s"
@@ -866,7 +871,7 @@ class AggregatedTimeSerie(TimeSerie):
             print("   Bytes per point: %.2f" % (len(s) / float(points)))
 
             t0 = time.time()
-            for i in six.moves.range(serialize_times):
+            for i in range(serialize_times):
                 cls.unserialize(s, key, 'mean')
             t1 = time.time()
             print("  Uncompression speed: %.2f MB/s"
@@ -877,7 +882,7 @@ class AggregatedTimeSerie(TimeSerie):
                 return 1 / ((t1 - t0) / serialize_times)
 
             t0 = time.time()
-            for i in six.moves.range(serialize_times):
+            for i in range(serialize_times):
                 list(ts.split())
             t1 = time.time()
             print("  split() speed: %.2f Hz" % per_sec(t1, t0))
@@ -891,7 +896,7 @@ class AggregatedTimeSerie(TimeSerie):
             )
 
             t0 = time.time()
-            for i in six.moves.range(serialize_times):
+            for i in range(serialize_times):
                 ts.merge(tsbis)
             t1 = time.time()
             print("  merge() speed %.2f Hz" % per_sec(t1, t0))
@@ -901,7 +906,7 @@ class AggregatedTimeSerie(TimeSerie):
                 serialize_times = 3 if agg.endswith('pct') else 10
                 ts = cls(ts=pts, aggregation=aggregation)
                 t0 = time.time()
-                for i in six.moves.range(serialize_times):
+                for i in range(serialize_times):
                     ts.resample(resample)
                 t1 = time.time()
                 print("  resample(%s) speed: %.2f Hz"
