@@ -17,9 +17,9 @@
 from __future__ import absolute_import
 
 from oslo_db.sqlalchemy import models
-import six
 import sqlalchemy
 from sqlalchemy.ext import declarative
+from sqlalchemy.orm import declarative_base
 import sqlalchemy_utils
 
 from gnocchi import archive_policy
@@ -28,7 +28,7 @@ from gnocchi.indexer import sqlalchemy_types as types
 from gnocchi import resource_type
 from gnocchi import utils
 
-Base = declarative.declarative_base()
+Base = declarative_base()
 
 COMMON_TABLES_ARGS = {'mysql_charset': "utf8",
                       'mysql_engine': "InnoDB"}
@@ -209,7 +209,7 @@ class ResourceJsonifier(indexer.Resource):
         d = dict(self)
         del d['revision']
         if 'metrics' not in sqlalchemy.inspect(self).unloaded:
-            d['metrics'] = dict((m.name, six.text_type(m.id))
+            d['metrics'] = dict((m.name, str(m.id))
                                 for m in self.metrics)
 
         if self.creator is None:
@@ -358,7 +358,7 @@ class HistoryModelIterator(models.ModelIterator):
         # NOTE(sileht): Our custom resource attribute columns don't
         # have the same name in database than in sqlalchemy model
         # so remove the additional "f_" for the model name
-        n = six.advance_iterator(self.i)
+        n = next(self.i)
         model_attr = n[2:] if n[:2] == "f_" else n
         return model_attr, getattr(self.model, n)
 
