@@ -24,8 +24,7 @@ import uuid
 import daiquiri
 import fixtures
 import numpy
-import six
-from six.moves.urllib.parse import unquote
+from urllib.parse import unquote
 try:
     from swiftclient import exceptions as swexc
 except ImportError:
@@ -59,7 +58,7 @@ def _skip_decorator(func):
         try:
             return func(*args, **kwargs)
         except exceptions.NotImplementedError as e:
-            raise testcase.TestSkipped(six.text_type(e))
+            raise testcase.TestSkipped(str(e))
     return skip_if_not_implemented
 
 
@@ -85,7 +84,7 @@ class FakeSwiftClient(object):
 
         files = []
         directories = set()
-        for k, v in six.iteritems(container.copy()):
+        for k, v in container.copy().items():
             if path and not k.startswith(path):
                 continue
 
@@ -216,12 +215,12 @@ class CaptureOutput(fixtures.Fixture):
 class BaseTestCase(testcase.TestCase):
     def setUp(self):
         super(BaseTestCase, self).setUp()
+
         if not os.getenv("GNOCCHI_TEST_DEBUG"):
             self.useFixture(CaptureOutput())
 
 
-@six.add_metaclass(SkipNotImplementedMeta)
-class TestCase(BaseTestCase):
+class TestCase(BaseTestCase, metaclass=SkipNotImplementedMeta):
 
     REDIS_DB_INDEX = 0
     REDIS_DB_LOCK = threading.Lock()
@@ -302,7 +301,7 @@ class TestCase(BaseTestCase):
             self.index.upgrade()
 
         self.archive_policies = self.ARCHIVE_POLICIES.copy()
-        for name, ap in six.iteritems(self.archive_policies):
+        for name, ap in self.archive_policies.items():
             # Create basic archive policies
             try:
                 self.index.create_archive_policy(ap)
@@ -315,7 +314,7 @@ class TestCase(BaseTestCase):
                                os.path.join(py_root, 'rest', 'api-paste.ini'),
                                group="api")
         self.conf.set_override('policy_file',
-                               os.path.join(py_root, 'rest', 'policy.json'),
+                               os.path.join(py_root, 'rest', 'policy.yaml'),
                                group="oslo_policy")
 
         # NOTE(jd) This allows to test S3 on AWS
@@ -323,9 +322,9 @@ class TestCase(BaseTestCase):
             self.conf.set_override('s3_endpoint_url',
                                    os.getenv("GNOCCHI_STORAGE_HTTP_URL"),
                                    group="storage")
-            self.conf.set_override('s3_access_key_id', "gnocchi",
+            self.conf.set_override('s3_access_key_id', "S3RVER",
                                    group="storage")
-            self.conf.set_override('s3_secret_access_key', "anythingworks",
+            self.conf.set_override('s3_secret_access_key', "S3RVER",
                                    group="storage")
 
         storage_driver = os.getenv("GNOCCHI_TEST_STORAGE_DRIVER", "file")
@@ -345,12 +344,17 @@ class TestCase(BaseTestCase):
             self.conf.set_override('ceph_conffile',
                                    os.getenv("CEPH_CONF"),
                                    'storage')
-            pool_name = uuid.uuid4().hex
+            self.ceph_pool_name = uuid.uuid4().hex
             with open(os.devnull, 'w') as f:
-                subprocess.call("rados -c %s mkpool %s" % (
-                    os.getenv("CEPH_CONF"), pool_name), shell=True,
+                subprocess.call(("ceph -c %s osd pool create %s "
+                                 "16 16 replicated") % (
+                    os.getenv("CEPH_CONF"), self.ceph_pool_name), shell=True,
                     stdout=f, stderr=subprocess.STDOUT)
-            self.conf.set_override('ceph_pool', pool_name, 'storage')
+                subprocess.call(("ceph -c %s osd pool application "
+                                 "enable %s rbd") % (
+                    os.getenv("CEPH_CONF"), self.ceph_pool_name), shell=True,
+                    stdout=f, stderr=subprocess.STDOUT)
+            self.conf.set_override('ceph_pool', self.ceph_pool_name, 'storage')
 
         # Override the bucket prefix to be unique to avoid concurrent access
         # with any other test
@@ -377,6 +381,15 @@ class TestCase(BaseTestCase):
     def tearDown(self):
         self.index.disconnect()
         self.coord.stop()
+
+        if self.conf.storage.driver == 'ceph':
+            with open(os.devnull, 'w') as f:
+                ceph_rmpool_command = "ceph -c %s osd pool delete %s %s \
+--yes-i-really-really-mean-it" % (os.getenv("CEPH_CONF"), self.ceph_pool_name,
+                                  self.ceph_pool_name)
+                subprocess.call(ceph_rmpool_command, shell=True,
+                                stdout=f, stderr=subprocess.STDOUT)
+
         super(TestCase, self).tearDown()
 
     def _create_metric(self, archive_policy_name="low"):
