@@ -16,11 +16,11 @@
 import datetime
 import functools
 import math
+import operator
 
 import fixtures
 import iso8601
 import numpy
-import six
 
 from gnocchi import carbonara
 from gnocchi.tests import base
@@ -104,21 +104,6 @@ class TestBoundTimeSerie(base.BaseTestCase):
 
 
 class TestAggregatedTimeSerie(base.BaseTestCase):
-    @staticmethod
-    def test_base():
-        carbonara.AggregatedTimeSerie.from_data(
-            3, 'mean',
-            [datetime64(2014, 1, 1, 12, 0, 0),
-             datetime64(2014, 1, 1, 12, 0, 4),
-             datetime64(2014, 1, 1, 12, 0, 9)],
-            [3, 5, 6])
-        carbonara.AggregatedTimeSerie.from_data(
-            "4s", 'mean',
-            [datetime64(2014, 1, 1, 12, 0, 0),
-             datetime64(2014, 1, 1, 12, 0, 4),
-             datetime64(2014, 1, 1, 12, 0, 9)],
-            [3, 5, 6])
-
     def test_benchmark(self):
         self.useFixture(fixtures.Timeout(300, gentle=True))
         carbonara.AggregatedTimeSerie.benchmark()
@@ -128,37 +113,28 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
             timestamps=[datetime64(2014, 1, 1, 12, 0, 0),
                         datetime64(2014, 1, 1, 12, 0, 4),
                         datetime64(2014, 1, 1, 12, 0, 9)],
-            aggregation_method='mean',
             values=[3, 5, 6],
-            sampling=numpy.timedelta64(1, 's'))
+            aggregation=carbonara.Aggregation(
+                "mean", numpy.timedelta64(1, 's'), None))
         self.assertEqual(
-            [(datetime64(2014, 1, 1, 12),
-              numpy.timedelta64(1000000, 'us'), 3),
-             (datetime64(2014, 1, 1, 12, 0, 4),
-              numpy.timedelta64(1000000, 'us'), 5),
-             (datetime64(2014, 1, 1, 12, 0, 9),
-              numpy.timedelta64(1000000, 'us'), 6)],
+            [(datetime64(2014, 1, 1, 12), 3),
+             (datetime64(2014, 1, 1, 12, 0, 4), 5),
+             (datetime64(2014, 1, 1, 12, 0, 9), 6)],
             list(ts.fetch()))
         self.assertEqual(
-            [(datetime64(2014, 1, 1, 12, 0, 4),
-              numpy.timedelta64(1000000, 'us'), 5),
-             (datetime64(2014, 1, 1, 12, 0, 9),
-              numpy.timedelta64(1000000, 'us'), 6)],
+            [(datetime64(2014, 1, 1, 12, 0, 4), 5),
+             (datetime64(2014, 1, 1, 12, 0, 9), 6)],
             list(ts.fetch(
                 from_timestamp=datetime64(2014, 1, 1, 12, 0, 4))))
         self.assertEqual(
-            [(datetime64(2014, 1, 1, 12, 0, 4),
-              numpy.timedelta64(1000000, 'us'), 5),
-             (datetime64(2014, 1, 1, 12, 0, 9),
-              numpy.timedelta64(1000000, 'us'), 6)],
+            [(datetime64(2014, 1, 1, 12, 0, 4), 5),
+             (datetime64(2014, 1, 1, 12, 0, 9), 6)],
             list(ts.fetch(
                 from_timestamp=numpy.datetime64(iso8601.parse_date(
                     "2014-01-01 12:00:04")))))
         self.assertEqual(
-            [(datetime64(2014, 1, 1, 12, 0, 4),
-              numpy.timedelta64(1000000, 'us'), 5),
-             (datetime64(2014, 1, 1, 12, 0, 9),
-              numpy.timedelta64(1000000, 'us'), 6)],
+            [(datetime64(2014, 1, 1, 12, 0, 4), 5),
+             (datetime64(2014, 1, 1, 12, 0, 9), 6)],
             list(ts.fetch(
                 from_timestamp=numpy.datetime64(iso8601.parse_date(
                     "2014-01-01 13:00:04+01:00")))))
@@ -175,11 +151,12 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
 
     @staticmethod
     def _resample(ts, sampling, agg, derived=False):
+        aggregation = carbonara.Aggregation(agg, sampling, None)
         grouped = ts.group_serie(sampling)
         if derived:
             grouped = grouped.derived()
         return carbonara.AggregatedTimeSerie.from_grouped_serie(
-            grouped, sampling, agg)
+            grouped, aggregation)
 
     def test_derived_mean(self):
         ts = carbonara.TimeSerie.from_data(
@@ -201,16 +178,11 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
 
         self.assertEqual(5, len(ts))
         self.assertEqual(
-            [(datetime64(2014, 1, 1, 12, 0, 0),
-              numpy.timedelta64(60, 's'), 5),
-             (datetime64(2014, 1, 1, 12, 1, 0),
-              numpy.timedelta64(60, 's'), 5),
-             (datetime64(2014, 1, 1, 12, 2, 0),
-              numpy.timedelta64(60, 's'), 11),
-             (datetime64(2014, 1, 1, 12, 3, 0),
-              numpy.timedelta64(60, 's'), -32),
-             (datetime64(2014, 1, 1, 12, 4, 0),
-              numpy.timedelta64(60, 's'), 16)],
+            [(datetime64(2014, 1, 1, 12, 0, 0), 5),
+             (datetime64(2014, 1, 1, 12, 1, 0), 5),
+             (datetime64(2014, 1, 1, 12, 2, 0), 11),
+             (datetime64(2014, 1, 1, 12, 3, 0), -32),
+             (datetime64(2014, 1, 1, 12, 4, 0), 16)],
             list(ts.fetch(
                 from_timestamp=datetime64(2014, 1, 1, 12))))
 
@@ -231,14 +203,10 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
 
         self.assertEqual(4, len(ts))
         self.assertEqual(
-            [(datetime64(2014, 1, 1, 12, 0, 0),
-              numpy.timedelta64(60, 's'), 5),
-             (datetime64(2014, 1, 1, 12, 1, 0),
-              numpy.timedelta64(60, 's'), 4),
-             (datetime64(2014, 1, 1, 12, 3, 0),
-              numpy.timedelta64(60, 's'), 92),
-             (datetime64(2014, 1, 1, 12, 4, 0),
-              numpy.timedelta64(60, 's'), 2)],
+            [(datetime64(2014, 1, 1, 12, 0, 0), 5),
+             (datetime64(2014, 1, 1, 12, 1, 0), 4),
+             (datetime64(2014, 1, 1, 12, 3, 0), 92),
+             (datetime64(2014, 1, 1, 12, 4, 0), 2)],
             list(ts.fetch(
                 from_timestamp=datetime64(2014, 1, 1, 12))))
 
@@ -257,7 +225,9 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
         key = ts.get_split_key()
         o, s = ts.serialize(key)
         saved_ts = carbonara.AggregatedTimeSerie.unserialize(
-            s, key, '74pct')
+            s, key, ts.aggregation)
+
+        self.assertEqual(ts.aggregation, saved_ts.aggregation)
 
         ts = carbonara.TimeSerie.from_data(
             [datetime64(2014, 1, 1, 12, 0, 0),
@@ -361,13 +331,14 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
                          ts[datetime64(2014, 1, 1, 12, 0, 0)][1])
 
     def test_different_length_in_timestamps_and_data(self):
-        self.assertRaises(ValueError,
-                          carbonara.AggregatedTimeSerie.from_data,
-                          3, 'mean',
-                          [datetime64(2014, 1, 1, 12, 0, 0),
-                           datetime64(2014, 1, 1, 12, 0, 4),
-                           datetime64(2014, 1, 1, 12, 0, 9)],
-                          [3, 5])
+        self.assertRaises(
+            ValueError,
+            carbonara.AggregatedTimeSerie.from_data,
+            carbonara.Aggregation('mean', numpy.timedelta64(3, 's'), None),
+            [datetime64(2014, 1, 1, 12, 0, 0),
+             datetime64(2014, 1, 1, 12, 0, 4),
+             datetime64(2014, 1, 1, 12, 0, 9)],
+            [3, 5])
 
     def test_truncate(self):
         ts = carbonara.TimeSerie.from_data(
@@ -426,11 +397,12 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
 
     @staticmethod
     def _resample_and_merge(ts, agg_dict):
-        """Helper method that mimics _add_measures workflow."""
+        """Helper method that mimics _compute_splits_operations workflow."""
         grouped = ts.group_serie(agg_dict['sampling'])
         existing = agg_dict.get('return')
         agg_dict['return'] = carbonara.AggregatedTimeSerie.from_grouped_serie(
-            grouped, agg_dict['sampling'], agg_dict['agg'])
+            grouped, carbonara.Aggregation(
+                agg_dict['agg'], agg_dict['sampling'], None))
         if existing:
             existing.merge(agg_dict['return'])
             agg_dict['return'] = existing
@@ -467,47 +439,28 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
                            self._resample_and_merge, agg_dict=ts))
 
         self.assertEqual([
-            (numpy.datetime64('2014-01-01T11:46:00.000000000'),
-             numpy.timedelta64(60, 's'), 4.0),
-            (numpy.datetime64('2014-01-01T11:47:00.000000000'),
-             numpy.timedelta64(60, 's'), 8.0),
-            (numpy.datetime64('2014-01-01T11:50:00.000000000'),
-             numpy.timedelta64(60, 's'), 50.0),
-            (datetime64(2014, 1, 1, 11, 54),
-             numpy.timedelta64(60000000000, 'ns'), 4.0),
-            (datetime64(2014, 1, 1, 11, 56),
-             numpy.timedelta64(60000000000, 'ns'), 4.0),
-            (datetime64(2014, 1, 1, 11, 57),
-             numpy.timedelta64(60000000000, 'ns'), 6.0),
-            (datetime64(2014, 1, 1, 11, 58),
-             numpy.timedelta64(60000000000, 'ns'), 5.0),
-            (datetime64(2014, 1, 1, 12, 1),
-             numpy.timedelta64(60000000000, 'ns'), 5.5),
-            (datetime64(2014, 1, 1, 12, 2),
-             numpy.timedelta64(60000000000, 'ns'), 8.0),
-            (datetime64(2014, 1, 1, 12, 3),
-             numpy.timedelta64(60000000000, 'ns'), 3.0),
-            (datetime64(2014, 1, 1, 12, 4),
-             numpy.timedelta64(60000000000, 'ns'), 7.0),
-            (datetime64(2014, 1, 1, 12, 5),
-             numpy.timedelta64(60000000000, 'ns'), 8.0),
-            (datetime64(2014, 1, 1, 12, 6),
-             numpy.timedelta64(60000000000, 'ns'), 4.0)
+            (numpy.datetime64('2014-01-01T11:46:00.000000000'), 4.0),
+            (numpy.datetime64('2014-01-01T11:47:00.000000000'), 8.0),
+            (numpy.datetime64('2014-01-01T11:50:00.000000000'), 50.0),
+            (datetime64(2014, 1, 1, 11, 54), 4.0),
+            (datetime64(2014, 1, 1, 11, 56), 4.0),
+            (datetime64(2014, 1, 1, 11, 57), 6.0),
+            (datetime64(2014, 1, 1, 11, 58), 5.0),
+            (datetime64(2014, 1, 1, 12, 1), 5.5),
+            (datetime64(2014, 1, 1, 12, 2), 8.0),
+            (datetime64(2014, 1, 1, 12, 3), 3.0),
+            (datetime64(2014, 1, 1, 12, 4), 7.0),
+            (datetime64(2014, 1, 1, 12, 5), 8.0),
+            (datetime64(2014, 1, 1, 12, 6), 4.0)
         ], list(ts['return'].fetch()))
 
         self.assertEqual([
-            (datetime64(2014, 1, 1, 12, 1),
-             numpy.timedelta64(60000000000, 'ns'), 5.5),
-            (datetime64(2014, 1, 1, 12, 2),
-             numpy.timedelta64(60000000000, 'ns'), 8.0),
-            (datetime64(2014, 1, 1, 12, 3),
-             numpy.timedelta64(60000000000, 'ns'), 3.0),
-            (datetime64(2014, 1, 1, 12, 4),
-             numpy.timedelta64(60000000000, 'ns'), 7.0),
-            (datetime64(2014, 1, 1, 12, 5),
-             numpy.timedelta64(60000000000, 'ns'), 8.0),
-            (datetime64(2014, 1, 1, 12, 6),
-             numpy.timedelta64(60000000000, 'ns'), 4.0)
+            (datetime64(2014, 1, 1, 12, 1), 5.5),
+            (datetime64(2014, 1, 1, 12, 2), 8.0),
+            (datetime64(2014, 1, 1, 12, 3), 3.0),
+            (datetime64(2014, 1, 1, 12, 4), 7.0),
+            (datetime64(2014, 1, 1, 12, 5), 8.0),
+            (datetime64(2014, 1, 1, 12, 6), 4.0)
         ], list(ts['return'].fetch(datetime64(2014, 1, 1, 12, 0, 0))))
 
     def test_fetch_agg_pct(self):
@@ -526,19 +479,18 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
         reference = [
             (datetime64(
                 2014, 1, 1, 12, 0, 0
-            ), 1.0, 3.9),
+            ), 3.9),
             (datetime64(
                 2014, 1, 1, 12, 0, 2
-            ), 1.0, 4)
+            ), 4)
         ]
 
         self.assertEqual(len(reference), len(list(result)))
 
         for ref, res in zip(reference, result):
             self.assertEqual(ref[0], res[0])
-            self.assertEqual(ref[1], res[1])
             # Rounding \o/
-            self.assertAlmostEqual(ref[2], res[2])
+            self.assertAlmostEqual(ref[1], res[1])
 
         tsb.set_values(numpy.array([
             (datetime64(2014, 1, 1, 12, 0, 2, 113), 110)],
@@ -550,19 +502,18 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
         reference = [
             (datetime64(
                 2014, 1, 1, 12, 0, 0
-            ), 1.0, 3.9),
+            ), 3.9),
             (datetime64(
                 2014, 1, 1, 12, 0, 2
-            ), 1.0, 99.4)
+            ), 99.4)
         ]
 
         self.assertEqual(len(reference), len(list(result)))
 
         for ref, res in zip(reference, result):
             self.assertEqual(ref[0], res[0])
-            self.assertEqual(ref[1], res[1])
             # Rounding \o/
-            self.assertAlmostEqual(ref[2], res[2])
+            self.assertAlmostEqual(ref[1], res[1])
 
     def test_fetch_nano(self):
         ts = {'sampling': numpy.timedelta64(200, 'ms'),
@@ -586,15 +537,13 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
                 self._resample_and_merge, agg_dict=ts))
 
         self.assertEqual([
-            (datetime64(2014, 1, 1, 11, 46, 0, 200000),
-             numpy.timedelta64(200000000, 'ns'), 6.0),
-            (datetime64(2014, 1, 1, 11, 47, 0, 200000),
-             numpy.timedelta64(200000000, 'ns'), 50.0),
-            (datetime64(2014, 1, 1, 11, 48, 0, 400000),
-             numpy.timedelta64(200000000, 'ns'), 4.0),
-            (datetime64(2014, 1, 1, 11, 48, 0, 800000),
-             numpy.timedelta64(200000000, 'ns'), 4.5)
+            (datetime64(2014, 1, 1, 11, 46, 0, 200000), 6.0),
+            (datetime64(2014, 1, 1, 11, 47, 0, 200000), 50.0),
+            (datetime64(2014, 1, 1, 11, 48, 0, 400000), 4.0),
+            (datetime64(2014, 1, 1, 11, 48, 0, 800000), 4.5)
         ], list(ts['return'].fetch()))
+        self.assertEqual(numpy.timedelta64(200000000, 'ns'),
+                         ts['return'].aggregation.granularity)
 
     def test_fetch_agg_std(self):
         # NOTE (gordc): this is a good test to ensure we drop NaN entries
@@ -613,10 +562,8 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
                            self._resample_and_merge, agg_dict=ts))
 
         self.assertEqual([
-            (datetime64(2014, 1, 1, 12, 1, 0),
-             numpy.timedelta64(60000000000, 'ns'), 2.1213203435596424),
-            (datetime64(2014, 1, 1, 12, 2, 0),
-             numpy.timedelta64(60000000000, 'ns'), 9.8994949366116654),
+            (datetime64(2014, 1, 1, 12, 1, 0), 2.1213203435596424),
+            (datetime64(2014, 1, 1, 12, 2, 0), 9.8994949366116654),
         ], list(ts['return'].fetch(datetime64(2014, 1, 1, 12, 0, 0))))
 
         tsb.set_values(numpy.array([(datetime64(2014, 1, 1, 12, 2, 13), 110)],
@@ -625,10 +572,8 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
                            self._resample_and_merge, agg_dict=ts))
 
         self.assertEqual([
-            (datetime64(2014, 1, 1, 12, 1, 0),
-             numpy.timedelta64(60000000000, 'ns'), 2.1213203435596424),
-            (datetime64(2014, 1, 1, 12, 2, 0),
-             numpy.timedelta64(60000000000, 'ns'), 59.304300012730948),
+            (datetime64(2014, 1, 1, 12, 1, 0), 2.1213203435596424),
+            (datetime64(2014, 1, 1, 12, 2, 0), 59.304300012730948),
         ], list(ts['return'].fetch(datetime64(2014, 1, 1, 12, 0, 0))))
 
     def test_fetch_agg_max(self):
@@ -646,12 +591,9 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
                            self._resample_and_merge, agg_dict=ts))
 
         self.assertEqual([
-            (datetime64(2014, 1, 1, 12, 0, 0),
-             numpy.timedelta64(60000000000, 'ns'), 3),
-            (datetime64(2014, 1, 1, 12, 1, 0),
-             numpy.timedelta64(60000000000, 'ns'), 7),
-            (datetime64(2014, 1, 1, 12, 2, 0),
-             numpy.timedelta64(60000000000, 'ns'), 15),
+            (datetime64(2014, 1, 1, 12, 0, 0), 3),
+            (datetime64(2014, 1, 1, 12, 1, 0), 7),
+            (datetime64(2014, 1, 1, 12, 2, 0), 15),
         ], list(ts['return'].fetch(datetime64(2014, 1, 1, 12, 0, 0))))
 
         tsb.set_values(numpy.array([(datetime64(2014, 1, 1, 12, 2, 13), 110)],
@@ -660,12 +602,9 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
                            self._resample_and_merge, agg_dict=ts))
 
         self.assertEqual([
-            (datetime64(2014, 1, 1, 12, 0, 0),
-             numpy.timedelta64(60, 's'), 3),
-            (datetime64(2014, 1, 1, 12, 1, 0),
-             numpy.timedelta64(60, 's'), 7),
-            (datetime64(2014, 1, 1, 12, 2, 0),
-             numpy.timedelta64(60, 's'), 110),
+            (datetime64(2014, 1, 1, 12, 0, 0), 3),
+            (datetime64(2014, 1, 1, 12, 1, 0), 7),
+            (datetime64(2014, 1, 1, 12, 2, 0), 110),
         ], list(ts['return'].fetch(datetime64(2014, 1, 1, 12, 0, 0))))
 
     def test_serialize(self):
@@ -686,13 +625,13 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
         o, s = ts['return'].serialize(key)
         self.assertEqual(ts['return'],
                          carbonara.AggregatedTimeSerie.unserialize(
-                             s, key, 'mean'))
+                             s, key, ts['return'].aggregation))
 
     def test_no_truncation(self):
         ts = {'sampling': numpy.timedelta64(60, 's'), 'agg': 'mean'}
         tsb = carbonara.BoundTimeSerie()
 
-        for i in six.moves.range(1, 11):
+        for i in range(1, 11):
             tsb.set_values(numpy.array([
                 (datetime64(2014, 1, 1, 12, i, i), float(i))],
                 dtype=carbonara.TIMESERIES_ARRAY_DTYPE),
@@ -726,15 +665,9 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
 
         self.assertEqual(
             [
-                (datetime64(
-                    2014, 1, 1, 12, 0, 1
-                ), numpy.timedelta64(1, 's'), 1.5),
-                (datetime64(
-                    2014, 1, 1, 12, 0, 2
-                ), numpy.timedelta64(1, 's'), 3.5),
-                (datetime64(
-                    2014, 1, 1, 12, 0, 3
-                ), numpy.timedelta64(1, 's'), 2.5),
+                (datetime64(2014, 1, 1, 12, 0, 1), 1.5),
+                (datetime64(2014, 1, 1, 12, 0, 2), 3.5),
+                (datetime64(2014, 1, 1, 12, 0, 3), 2.5),
             ],
             list(ts['return'].fetch()))
 
@@ -759,12 +692,9 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
 
         self.assertEqual(
             [
-                (datetime64(2014, 1, 1, 12, 0, 1),
-                 numpy.timedelta64(1, 's'), 1.5),
-                (datetime64(2014, 1, 1, 12, 0, 2),
-                 numpy.timedelta64(1, 's'), 3.5),
-                (datetime64(2014, 1, 1, 12, 0, 3),
-                 numpy.timedelta64(1, 's'), 2.5),
+                (datetime64(2014, 1, 1, 12, 0, 1), 1.5),
+                (datetime64(2014, 1, 1, 12, 0, 2), 3.5),
+                (datetime64(2014, 1, 1, 12, 0, 3), 2.5),
             ],
             list(ts['return'].fetch()))
 
@@ -776,12 +706,9 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
 
         self.assertEqual(
             [
-                (datetime64(2014, 1, 1, 12, 0, 1),
-                 numpy.timedelta64(1, 's'), 1.5),
-                (datetime64(2014, 1, 1, 12, 0, 2),
-                 numpy.timedelta64(1, 's'), 3.5),
-                (datetime64(2014, 1, 1, 12, 0, 3),
-                 numpy.timedelta64(1, 's'), 2.5),
+                (datetime64(2014, 1, 1, 12, 0, 1), 1.5),
+                (datetime64(2014, 1, 1, 12, 0, 2), 3.5),
+                (datetime64(2014, 1, 1, 12, 0, 3), 2.5),
             ],
             list(ts['return'].fetch()))
 
@@ -794,12 +721,9 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
 
         self.assertEqual(
             [
-                (datetime64(2014, 1, 1, 12, 0, 1),
-                 numpy.timedelta64(1, 's'), 1.5),
-                (datetime64(2014, 1, 1, 12, 0, 2),
-                 numpy.timedelta64(1, 's'), 3.5),
-                (datetime64(2014, 1, 1, 12, 0, 3),
-                 numpy.timedelta64(1, 's'), 3.5),
+                (datetime64(2014, 1, 1, 12, 0, 1), 1.5),
+                (datetime64(2014, 1, 1, 12, 0, 2), 3.5),
+                (datetime64(2014, 1, 1, 12, 0, 3), 3.5),
             ],
             list(ts['return'].fetch()))
 
@@ -828,6 +752,7 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
         dt1_1 = numpy.datetime64("2015-01-01T15:03")
         dt2 = numpy.datetime64("2015-01-05T15:03")
         td = numpy.timedelta64(60, 's')
+        td2 = numpy.timedelta64(300, 's')
 
         self.assertEqual(
             carbonara.SplitKey.from_timestamp_and_sampling(dt1, td),
@@ -838,6 +763,9 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
         self.assertNotEqual(
             carbonara.SplitKey.from_timestamp_and_sampling(dt1, td),
             carbonara.SplitKey.from_timestamp_and_sampling(dt2, td))
+        self.assertNotEqual(
+            carbonara.SplitKey.from_timestamp_and_sampling(dt1, td),
+            carbonara.SplitKey.from_timestamp_and_sampling(dt1, td2))
 
         self.assertLess(
             carbonara.SplitKey.from_timestamp_and_sampling(dt1, td),
@@ -858,6 +786,7 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
         dt1_1 = numpy.datetime64("2015-01-01T15:03")
         dt2 = numpy.datetime64("2015-01-05T15:03")
         td = numpy.timedelta64(60, 's')
+        td2 = numpy.timedelta64(300, 's')
 
         self.assertFalse(
             carbonara.SplitKey.from_timestamp_and_sampling(dt1, td) !=
@@ -868,6 +797,29 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
         self.assertFalse(
             carbonara.SplitKey.from_timestamp_and_sampling(dt1, td) ==
             carbonara.SplitKey.from_timestamp_and_sampling(dt2, td))
+        self.assertFalse(
+            carbonara.SplitKey.from_timestamp_and_sampling(dt1, td) ==
+            carbonara.SplitKey.from_timestamp_and_sampling(dt2, td2))
+        self.assertRaises(
+            TypeError,
+            operator.le,
+            carbonara.SplitKey.from_timestamp_and_sampling(dt1, td),
+            carbonara.SplitKey.from_timestamp_and_sampling(dt2, td2))
+        self.assertRaises(
+            TypeError,
+            operator.ge,
+            carbonara.SplitKey.from_timestamp_and_sampling(dt1, td),
+            carbonara.SplitKey.from_timestamp_and_sampling(dt2, td2))
+        self.assertRaises(
+            TypeError,
+            operator.gt,
+            carbonara.SplitKey.from_timestamp_and_sampling(dt1, td),
+            carbonara.SplitKey.from_timestamp_and_sampling(dt2, td2))
+        self.assertRaises(
+            TypeError,
+            operator.lt,
+            carbonara.SplitKey.from_timestamp_and_sampling(dt1, td),
+            carbonara.SplitKey.from_timestamp_and_sampling(dt2, td2))
 
         self.assertFalse(
             carbonara.SplitKey.from_timestamp_and_sampling(dt1, td) >=
@@ -900,8 +852,8 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
         points = 100000
         ts = carbonara.TimeSerie.from_data(
             timestamps=list(map(datetime.datetime.utcfromtimestamp,
-                                six.moves.range(points))),
-            values=list(six.moves.range(points)))
+                                range(points))),
+            values=list(range(points)))
         agg = self._resample(ts, sampling, 'mean')
 
         grouped_points = list(agg.split())
@@ -923,17 +875,15 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
         points = 100000
         ts = carbonara.TimeSerie.from_data(
             timestamps=list(map(datetime.datetime.utcfromtimestamp,
-                                six.moves.range(points))),
-            values=list(six.moves.range(points)))
+                                range(points))),
+            values=list(range(points)))
         agg = self._resample(ts, sampling, 'mean')
 
         split = [t[1] for t in list(agg.split())]
 
         self.assertEqual(agg,
                          carbonara.AggregatedTimeSerie.from_timeseries(
-                             split,
-                             sampling=agg.sampling,
-                             aggregation_method=agg.aggregation_method))
+                             split, aggregation=agg.aggregation))
 
     def test_resample(self):
         ts = carbonara.TimeSerie.from_data(
@@ -950,3 +900,15 @@ class TestAggregatedTimeSerie(base.BaseTestCase):
         self.assertEqual(2, len(agg_ts))
         self.assertEqual(5, agg_ts[0][1])
         self.assertEqual(3, agg_ts[1][1])
+
+    def test_iter(self):
+        ts = carbonara.TimeSerie.from_data(
+            [datetime64(2014, 1, 1, 12, 0, 0),
+             datetime64(2014, 1, 1, 12, 0, 11),
+             datetime64(2014, 1, 1, 12, 0, 12)],
+            [3, 5, 6])
+        self.assertEqual([
+            (numpy.datetime64('2014-01-01T12:00:00'), 3.),
+            (numpy.datetime64('2014-01-01T12:00:11'), 5.),
+            (numpy.datetime64('2014-01-01T12:00:12'), 6.),
+        ], list(ts))
