@@ -26,10 +26,9 @@ import uuid
 import fixtures
 import iso8601
 from keystonemiddleware import fixture as ksm_fixture
-import mock
-import six
 import testscenarios
 from testtools import testcase
+from unittest import mock
 import webtest
 
 import gnocchi
@@ -295,7 +294,7 @@ class ArchivePolicyTest(RestTest):
         # Transform list to set
         for ap in aps:
             ap['aggregation_methods'] = set(ap['aggregation_methods'])
-        for name, ap in six.iteritems(self.archive_policies):
+        for name, ap in self.archive_policies.items():
             apj = ap.jsonify()
             apj['definition'] = [
                 archive_policy.ArchivePolicyItem(**d).jsonify()
@@ -389,7 +388,8 @@ class MetricTest(RestTest):
                                     params={"archive_policy_name": "medium"})
         metric = json.loads(result.text)
         with self.app.use_another_user():
-            self.app.delete("/v1/metric/" + metric['id'], status=403)
+            r = self.app.delete("/v1/metric/" + metric['id'], status=403)
+            self.assertEqual("text/plain", r.content_type)
 
     def test_add_measure_with_another_user(self):
         result = self.app.post_json("/v1/metric",
@@ -976,18 +976,20 @@ class ResourceTest(RestTest):
         self.attributes['metrics'] = {'foo': {'archive_policy_name': "high"}}
         self.app.post_json("/v1/resource/" + self.resource_type,
                            params=self.attributes)
-        self.app.delete("/v1/resource/"
-                        + self.resource_type
-                        + "/"
-                        + self.attributes['id']
-                        + "/metric/foo",
-                        status=204)
-        self.app.delete("/v1/resource/"
-                        + self.resource_type
-                        + "/"
-                        + self.attributes['id']
-                        + "/metric/foo/measures",
-                        status=404)
+        r = self.app.delete("/v1/resource/"
+                            + self.resource_type
+                            + "/"
+                            + self.attributes['id']
+                            + "/metric/foo",
+                            status=204)
+        self.assertEqual(None, r.content_type)
+        r = self.app.delete("/v1/resource/"
+                            + self.resource_type
+                            + "/"
+                            + self.attributes['id']
+                            + "/metric/foo/measures",
+                            status=404)
+        self.assertEqual("text/plain", r.content_type)
 
     def test_get_resource_unknown_named_metric(self):
         self.app.post_json("/v1/resource/" + self.resource_type,
@@ -1141,7 +1143,7 @@ class ResourceTest(RestTest):
         result = json.loads(response.text)
         presult = json.loads(presponse.text)
         self.assertEqual(result, presult)
-        for k, v in six.iteritems(self.patchable_attributes):
+        for k, v in self.patchable_attributes.items():
             self.assertEqual(v, result[k])
         self.assertIsNone(result['revision_end'])
         self.assertEqual(result['revision_start'],
@@ -1159,7 +1161,7 @@ class ResourceTest(RestTest):
         self.assertEqual(result, history[1])
 
         h = history[0]
-        for k, v in six.iteritems(self.attributes):
+        for k, v in self.attributes.items():
             self.assertEqual(v, h[k])
         self.assertEqual(h['revision_end'],
                          "2014-01-02T06:48:00+00:00")
@@ -1230,10 +1232,15 @@ class ResourceTest(RestTest):
             "/v1/resource/" + self.resource_type + "/"
             + self.attributes['id'],
             params={'foobar': 123},
-            status=400)
-        self.assertIn(b'Invalid input: extra keys not allowed @ data['
-                      + repr(u'foobar').encode('ascii') + b"]",
-                      result.body)
+            status=400,
+            headers={"Accept": "application/json"}
+        )
+
+        result_description = result.json['description']
+        self.assertEqual("Invalid input", result_description['cause'])
+        self.assertIn(
+            "extra keys not allowed @ data[", result_description['reason']
+        )
 
     def test_delete_resource(self):
         self.app.post_json("/v1/resource/" + self.resource_type,
@@ -1241,9 +1248,10 @@ class ResourceTest(RestTest):
         self.app.get("/v1/resource/" + self.resource_type + "/"
                      + self.attributes['id'],
                      status=200)
-        self.app.delete("/v1/resource/" + self.resource_type + "/"
-                        + self.attributes['id'],
-                        status=204)
+        r = self.app.delete("/v1/resource/" + self.resource_type + "/"
+                            + self.attributes['id'],
+                            status=204)
+        self.assertEqual(None, r.content_type)
         self.app.get("/v1/resource/" + self.resource_type + "/"
                      + self.attributes['id'],
                      status=404)
@@ -1253,7 +1261,7 @@ class ResourceTest(RestTest):
             "/v1/metric",
             params={'archive_policy_name': "high"})
         metric_id = json.loads(metric.text)['id']
-        metric_name = six.text_type(uuid.uuid4())
+        metric_name = str(uuid.uuid4())
         self.attributes['metrics'] = {metric_name: metric_id}
         self.app.get("/v1/metric/" + metric_id,
                      status=200)
@@ -1262,9 +1270,10 @@ class ResourceTest(RestTest):
         self.app.get("/v1/resource/" + self.resource_type + "/"
                      + self.attributes['id'],
                      status=200)
-        self.app.delete("/v1/resource/" + self.resource_type + "/"
-                        + self.attributes['id'],
-                        status=204)
+        r = self.app.delete("/v1/resource/" + self.resource_type + "/"
+                            + self.attributes['id'],
+                            status=204)
+        self.assertEqual(None, r.content_type)
         self.app.get("/v1/resource/" + self.resource_type + "/"
                      + self.attributes['id'],
                      status=404)
@@ -1275,14 +1284,16 @@ class ResourceTest(RestTest):
         self.app.post_json("/v1/resource/" + self.resource_type,
                            params=self.attributes)
         with self.app.use_another_user():
-            self.app.delete("/v1/resource/" + self.resource_type + "/"
-                            + self.attributes['id'],
-                            status=403)
+            r = self.app.delete("/v1/resource/" + self.resource_type + "/"
+                                + self.attributes['id'],
+                                status=403)
+            self.assertEqual("text/plain", r.content_type)
 
     def test_delete_resource_non_existent(self):
         result = self.app.delete("/v1/resource/" + self.resource_type + "/"
                                  + self.attributes['id'],
                                  status=404)
+        self.assertEqual("text/plain", result.content_type)
         self.assertIn(
             "Resource %s does not exist" % self.attributes['id'],
             result.text)
@@ -1864,11 +1875,15 @@ class GenericResourceTest(RestTest):
         result = self.app.post_json(
             "/v1/search/resource/generic",
             params={"wrongoperator": {"user_id": "bar"}},
-            status=400)
+            status=400,
+            headers={"Accept": "application/json"},
+        )
+
+        result_description = result.json['description']
+        self.assertEqual("Invalid input", result_description['cause'])
         self.assertIn(
-            "Invalid input: extra keys not allowed @ data["
-            + repr(u'wrongoperator') + "]",
-            result.text)
+            "extra keys not allowed @ data[", result_description['reason']
+        )
 
 
 class QueryStringSearchAttrFilterTest(tests_base.TestCase):
